import RxSwift
import PlaygroundSupport
PlaygroundPage.current.needsIndefiniteExecution = true

// Создание наблюдаемых объектов для предопределенной последовательности

let justObservable = Observable<Int>.just(1)

let ofObservable = Observable.of(1, 2, 3, 4, 5)

let fromObservable = Observable.from([1, 2, 3, 4, 5])

let rangeObservable = Observable.range(start: 1, count: 5)

// Создание наблюдаемого объекта для URLSession при помощи .create

let urlObservable = Observable<String>.create { observer in
    let defaultSession = URLSession(configuration: .default)
    let url = URL(string: "https://api.github.com")!
    
    let dataTask = defaultSession.dataTask(with: url) { data, response, error in
        if let error = error {
            observer.on(.error(error))
        } else if let data = data {
            let responseString = String(decoding: data, as: UTF8.self)
            observer.on(.next(responseString))
            observer.on(.completed)
        }
    }
    
    dataTask.resume()
    
    return Disposables.create {
        dataTask.cancel()
    }
}

// Пример deferred
var i = 1
let plainObservable = Observable<Int>.just(i)
let deferredObservable = Observable.deferred{ return Observable<Int>.just(i) }
i += 1
plainObservable.subscribe(onNext: { i in print(i) }) // 1
deferredObservable.subscribe(onNext: { i in print(i) }) // 2

// Подписываемся на получение событий

let disposeBag = DisposeBag()

urlObservable.subscribe(
    onNext: { (responseString) in
        print(responseString)
},
    onError: { (err) in
        print("ERROR: \(err.localizedDescription)")
},
    onCompleted: {
        print("URL observable completed")
},
    onDisposed: {
        print("URL observable disposed")
}).disposed(by: disposeBag)

rangeObservable.subscribe { (event) in
    print(event)
}.disposed(by: disposeBag)

fromObservable.subscribe(
    onNext: { (number) in
        print(number)
},
    onError: { (err) in
        print(err.localizedDescription)
},
    onCompleted: {
        print("Completed")
},
    onDisposed: {
        print("Disposed")
}).disposed(by: disposeBag)

// Более короткий вариант, если не нужно обрабатывать другие события:

fromObservable.subscribe(
    onNext: { (number) in
        print(number)
}).disposed(by: disposeBag)

// Traits, пример с URLSession и Single

let urlSingle = Single<String>.create { single in
    let defaultSession = URLSession(configuration: .default)
    let url = URL(string: "https://api.github.com")!
    
    let dataTask = defaultSession.dataTask(with: url) { data, response, error in
        if let error = error {
            single(.error(error))
        } else if let data = data {
            let responseString = String(decoding: data, as: UTF8.self)
            single(.success(responseString))
        }
    }
    
    dataTask.resume()
    
    return Disposables.create {
        dataTask.cancel()
    }
}

urlSingle.subscribe(
    onSuccess: { (response) in
        print("SUCCESS: \(response)")
},
    onError: { (error) in
        print("ERROR: \(error.localizedDescription)")
})
